/*-
 * Copyright 2016 UPLEX - Nils Goroll Systemoptimierung
 * All rights reserved.
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "config.h"

#include <string.h>

#include "vcl.h"
#include "vas.h"
#include "vdef.h"
#include "vrt.h"
#include "cache/cache.h"
#include "vsb.h"

#include "vcc_if.h"

#define ERR(ctx, msg) \
        errmsg((ctx), "vmod blob error: " msg)

#define VERR(ctx, fmt, ...) \
        errmsg((ctx), "vmod blob error: " fmt, __VA_ARGS__)

static void
errmsg(VRT_CTX, const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	if (ctx->vsl)
		VSLbv(ctx->vsl, SLT_VCL_Error, fmt, args);
	else
		VSLv(SLT_VCL_Error, 0, fmt, args);
	va_end(args);

	if (ctx->method == VCL_MET_INIT) {
		AN(ctx->msg);
		va_start(args, fmt);
		VSB_vprintf(ctx->msg, fmt, args);
		va_end(args);
		VRT_handling(ctx, VCL_RET_FAIL);
	}
}

VCL_BOOL
vmod_same(VRT_CTX __attribute__((unused)), VCL_BLOB b1, VCL_BLOB b2)
{
	return b1->len == b2->len && b1->priv == b2->priv;
}

VCL_BOOL
vmod_equal(VRT_CTX __attribute__((unused)), VCL_BLOB b1, VCL_BLOB b2)
{
	if (b1->len != b2->len)
		return 0;
	if (b1->priv == b2->priv)
		return 1;
	return (memcmp(b1->priv, b2->priv, b1->len) == 0);
}

VCL_INT
vmod_length(VRT_CTX __attribute__((unused)), VCL_BLOB b)
{
	return b->len;
}

VCL_INT
vmod_integer(VRT_CTX, VCL_BLOB b, VCL_ENUM padding)
{
	VCL_INT i = 0;

	assert(b->len >= 0);
	if ((unsigned)b->len >= sizeof(VCL_INT))
		return *((VCL_INT *) b->priv);
	if (b->len == 0) {
		ERR(ctx, "blob length is 0 in blob.integer()");
		return 0;
	}
	for (int j = 0; j < b->len; j++)
		i |= ((unsigned char *)b->priv)[j] << ((b->len - j - 1) * 8);
	if (padding[0] == 'R')
		i <<= (sizeof(VCL_INT) - b->len) * 8;
	return i;
}

VCL_STRING
vmod_version(VRT_CTX __attribute__((unused)))
{
	return VERSION;
}
